package com.qucheng.game.data.oldsystem.serialization;

import com.qucheng.game.data.oldsystem.pojo.TransportMap;
import com.qucheng.game.data.oldsystem.util.CDCStructUtil;
import com.ververica.cdc.debezium.DebeziumDeserializationSchema;
import io.debezium.data.Envelope;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.api.common.typeinfo.Types;
import org.apache.flink.util.Collector;
import org.apache.kafka.connect.data.Field;
import org.apache.kafka.connect.data.Struct;
import org.apache.kafka.connect.source.SourceRecord;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class MapDebeziumDeserializationSchema implements DebeziumDeserializationSchema<TransportMap> {

    @Override
    public void deserialize(SourceRecord sourceRecord, Collector<TransportMap> collector) {
        //解析库名&表名
        String[] topics = sourceRecord.topic().split("\\.");
        String dbName = topics[1];
        String tableName = topics[2];

        List<String> primaryKeys = null;
        //解析主键
        Struct key = (Struct) sourceRecord.key();
        if (key != null) {
            primaryKeys = new ArrayList<>(key.schema().fields().size());
            for (Field field : key.schema().fields()) {
                primaryKeys.add(field.name());
            }
        }

        // 解析值
        Struct struct = (Struct) sourceRecord.value();
        Struct after = struct.getStruct("after");
        Struct before = struct.getStruct("before");
        //获取操作类型
        Envelope.Operation operation = Envelope.operationFor(sourceRecord);

        Map<String, Object> beforeBean = null;
        if (before != null) {
            beforeBean = CDCStructUtil.structToMap(before);
        }
        Map<String, Object> afterBean = null;
        if (after != null) {
            afterBean = CDCStructUtil.structToMap(after);
        }
        collector.collect(TransportMap.builder()
                .dbName(dbName)
                .tableName(tableName)
                .primaryKeys(primaryKeys)
                .operation(operation)
                .before(beforeBean)
                .after(afterBean)
                .build()
        );
    }

    @Override
    public TypeInformation<TransportMap> getProducedType() {
        return Types.POJO(TransportMap.class);
    }
}
