package com.qucheng.game.data.oldsystem.sink;

import com.qucheng.game.data.oldsystem.dao.mapper.BaseMapper;
import com.qucheng.game.data.oldsystem.pojo.MysqlConfigParam;
import com.qucheng.game.data.oldsystem.pojo.TransportMap;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.apache.ibatis.mapping.Environment;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.apache.ibatis.transaction.jdbc.JdbcTransactionFactory;

import javax.sql.DataSource;

public class MpConfigAgentSink extends RichSinkFunction<Tuple3<String, String, TransportMap>> {
    private final MysqlConfigParam bgNewMysqlConfigParam;
    private final MysqlConfigParam bgOldMysqlConfigParam;
    private final MysqlConfigParam qcMysqlConfigParam;
    protected SqlSessionFactory bgNewSqlSessionFactory;
    protected SqlSessionFactory bgOldSqlSessionFactory;
    protected SqlSessionFactory qcSqlSessionFactory;

    public MpConfigAgentSink(MysqlConfigParam bgNewMysqlConfigParam, MysqlConfigParam bgOldMysqlConfigParam, MysqlConfigParam qcMysqlConfigParam) {
        this.bgNewMysqlConfigParam = bgNewMysqlConfigParam;
        this.bgOldMysqlConfigParam = bgOldMysqlConfigParam;
        this.qcMysqlConfigParam = qcMysqlConfigParam;
    }

    @Override
    public void open(Configuration configuration) {
        bgNewSqlSessionFactory = sqlSessionFactory(bgNewMysqlConfigParam);
        bgOldSqlSessionFactory = sqlSessionFactory(bgOldMysqlConfigParam);
        qcSqlSessionFactory = sqlSessionFactory(qcMysqlConfigParam);
    }

    @Override
    public void invoke(Tuple3<String, String, TransportMap> tuple3, Context context) {
        switch (tuple3.f0) {
            case "mp_conf_agent":
                invokeMpConfigAgent(tuple3.f1, tuple3.f2);
                break;
            case "ad_account":
                invokeAdAccount(tuple3.f1, tuple3.f2);
                break;
            default:
                break;
        }
    }

    private void invokeMpConfigAgent(String sdkSource, TransportMap value) {
        if (sdkSource.equals("BG_NEW")) {
            try (SqlSession session = bgNewSqlSessionFactory.openSession(true)) {
                BaseMapper mapper = session.getMapper(BaseMapper.class);
                mapper.saveOrUpdate(value.getTableName(), value.getPrimaryKeys(), value.getAfter());
            }
        } else {
            try (SqlSession session = bgOldSqlSessionFactory.openSession(true)) {
                BaseMapper mapper = session.getMapper(BaseMapper.class);
                mapper.saveOrUpdate(value.getTableName(), value.getPrimaryKeys(), value.getAfter());
            }
        }
    }

    private void invokeAdAccount(String sourceSystem, TransportMap value) {
        try (SqlSession session = qcSqlSessionFactory.openSession(true)) {
            BaseMapper mapper = session.getMapper(BaseMapper.class);
            mapper.saveOrUpdate(value.getTableName(), value.getPrimaryKeys(), value.getAfter());
        }
    }

    @Override
    public void close() {
    }

    private SqlSessionFactory sqlSessionFactory(MysqlConfigParam mysqlConfig) {
        // 初始化连接池
        HikariConfig config = new HikariConfig();
        config.setDriverClassName(mysqlConfig.getDriverClassName());
        config.setJdbcUrl(mysqlConfig.getUrl());
        config.setUsername(mysqlConfig.getUsername());
        config.setPassword(mysqlConfig.getPassword());
        //连接池内保留的最少连接数
        config.setMinimumIdle(1);
        DataSource dataSource = new HikariDataSource(config);

        Environment mybatisEnv = new Environment("ads-mysql", new JdbcTransactionFactory(), dataSource);

        org.apache.ibatis.session.Configuration mybatisConfig = new org.apache.ibatis.session.Configuration(mybatisEnv);
        mybatisConfig.addMapper(BaseMapper.class);
        return new SqlSessionFactoryBuilder().build(mybatisConfig);
    }
}
